from ._version import get_versions
__version__ = get_versions()['version']
del get_versions

import os
import sys

# operating system specific imports
if os.name == 'nt':
    from .windows import ShortCutterWindows as ShortCutter
elif os.name == 'posix':
    if sys.platform == 'darwin':
        from .macos import ShortCutterMacOS as ShortCutter
    else:
        from .linux import ShortCutterLinux as ShortCutter
else:
    raise Exception("Error: '{}' platform is not supported.".format(sys.platform))


def main():
    from argparse import ArgumentParser

    parser = ArgumentParser(description="Automatic shortcut creator." +
                                        " Shortcuts auto-activate their environments by default.")
    parser.add_argument("target", nargs='?', default=None,
                        help="The target executable to create Desktop and Menu shortcuts.")
    parser.add_argument("-d", "--desktop", action="store_true", help="Only create a desktop shortcut.")
    parser.add_argument("-m", "--menu", action="store_true", help="Only create a menu shortcut.")
    parser.add_argument("-n", "--name", nargs='?', default=None, help="Name of the shortcut without extension (autoname otherwise).")
    parser.add_argument("-s", "--simple", action="store_true", help="Create simple shortcut without activate wrapper.")
    parser.add_argument("-t", "--terminal", action="store_true",
                        help="Create shortcut to environment with shortcutter " +
                             "(plus shortcut to root environment in case of conda).")
    args = parser.parse_args()

    create_desktop = args.desktop
    create_menu = args.menu
    activate = not args.simple
    name = args.name if args.name else None

    if not args.target and not args.terminal:
        print('Shortcutter needs target or --terminal arguments to work.')
        return

    # if desktop or menu hasnt been specified create both (i.e. the default)
    if not create_desktop and not create_menu:
        create_desktop = True
        create_menu = True

    sc = ShortCutter(activate=activate, error_log=sys.stdout)

    target_path = sc.find_target(args.target)
    if target_path or args.terminal:

        desktop_created = False
        if create_desktop:
            if args.terminal:
                desktop_created = sc.create_shortcut_to_env_terminal(name, menu=False)
            else:
                ret = sc.create_desktop_shortcut(target_path, name)
                desktop_created = bool(ret[2])  # shortcut_path = ret[2]
            if not desktop_created:
                print("Failed to create desktop shortcut.")

        menu_created = False
        if create_menu:
            if args.terminal:
                menu_created = sc.create_shortcut_to_env_terminal(name, desktop=False)
            else:
                ret = sc.create_menu_shortcut(target_path, name)
                menu_created = bool(ret[2])  # shortcut_path = ret[2]
            if not menu_created:
                print("Failed to create menu shortcut.")

        msg = "created for '{}'.".format(args.target if not args.terminal else 'terminal at environment')
        if desktop_created and menu_created:
            print('Desktop and menu shortcuts were ' + msg)
        elif desktop_created and not menu_created:
            print('Desktop shortcut was ' + msg)
        elif not desktop_created and menu_created:
            print('Menu shortcut was ' + msg)
    else:
        print("Shortcut creation failed: unable to find '{}'.".format(args.target))



import os
from os import path as p
from .exception import ShortcutError, ShortcutNoDesktopError, ShortcutNoMenuError
import re
import traceback


class ShortCutter(object):
    """
    Creates applicaton shortcuts for Windows, MacOS and Linux operating systems.

    To create desktop and menu shortcuts to ``python``::

        from shortcutter import ShortCutter
        s = ShortCutter()
        s.create_desktop_shortcut("python")
        s.create_menu_shortcut("python")

    Attributes:
    -----------
    raise_errors : bool=False
        Whether to raise exceptions or skip errors and continue.
    error_log : object=None
        File object where to write errors when ``raise_errors=False``.
        Default is None - do not write errors.
        Can also be ``sys.stderr`` or ``io.StringIO()``.
    desktop_folder : str
        Directory used when creating desktop shortcuts.
    menu_folder : str
        Directory used when creating menu shortcuts.
    bin_folder_pyexe : str
        ``Scripts`` or ``bin`` dir path. Simply closest to python executable path.
    bin_folder_shcut : str or None
        ``Scripts`` or ``bin`` dir path where shortcutter executable was installed.
    local_root : str
        Root directory path of the current python environment / installation.
        Derived from python executable path.
    activate : bool=True
        Whether to create shortcuts that automatically activate
        conda environment / virtual environment.
    exists : bool=True
        Whether the target should exist or not.
        If not then add ``/`` (``\\`` on Windows) at the end of the path to get dir shortcut.
    activate_args : tuple (str or None, str or None)
        First is the activate script full path (or None if it's wasn't found) - conda's or venv's.
        Second is the env argument of the activate script (or None if not needed).
    """

    def __init__(self, raise_errors=False, error_log=None, activate=True, exists=True):
        """
        Creates ShortCutter.

        Parameters
        ----------
        raise_errors : bool=False
            Whether to raise exceptions or skip errors and continue.
        error_log : object=None
            File object where to write errors when ``raise_errors=False``.
            Default is None - do not write errors.
            Can also be ``sys.stderr`` or ``io.StringIO()``.
        activate : bool=True
            Whether to create shortcuts that automatically activate
            conda environment / virtual environment.
        exists : bool=True
            Whether the target should exist or not.
            If not then add ``/`` (``\\`` on Windows) at the end of the path to get dir shortcut.
        """
        self._set_win_vars()  # important on Windows

        self.raise_errors = raise_errors
        self.error_log = error_log
        self.activate = activate
        self.exists = exists
        self.desktop_folder = self._get_desktop_folder()
        self.menu_folder = self._get_menu_folder()
        self.bin_folder_pyexe = self._get_bin_folder_pyexe()
        shortcutter = self.find_target('shortcutter')
        self.bin_folder_shcut = p.dirname(shortcutter) if shortcutter else self.bin_folder_pyexe
        self.local_root = self._get_local_root()
        self._ACTIVATE, self._ACTIVATE_PROMPT = self._get_activate_wrapper_templates()

        self.activate_args = self._get_activate_args()  # should be run the last

    # might be overridden if needed
    def _set_win_vars(self):
        pass

    # should be overridden
    @staticmethod
    def _get_desktop_folder():
        raise ShortcutError("_get_desktop_folder needs overriding")

    # should be overridden
    @staticmethod
    def _get_menu_folder():
        raise ShortcutError("_get_menu_folder needs overriding")

    # should be overridden
    @staticmethod
    def _get_bin_folder_pyexe():
        raise ShortcutError("_get_bin_folder_pyexe needs overriding")

    # should be overridden
    @staticmethod
    def _get_local_root():
        raise ShortcutError("_get_local_root needs overriding")

    # should be overridden
    @staticmethod
    def _get_activate_wrapper_templates():
        raise ShortcutError("_get_activate_wrapper_templates needs overriding")

    # should be overridden
    @staticmethod
    def _make_executable(file_path):
        raise ShortcutError("_make_executable needs overriding")

    def exe(self, app_name):
        """
        Returns platform independent executable name:

        * app > app (on Unix)
        * app > app.exe (on Windows)
        """
        if os.name == 'nt':
            return app_name + '.exe'
        else:
            return app_name

    def ba(self, script_name):
        """
        Returns platform independent shell script (bash/batch) name:

        * run > run (on Unix)
        * run > run.bat (on Windows)
        """
        if os.name == 'nt':
            return script_name + '.bat'
        else:
            return script_name

    def _get_activate_args(self):
        """
        Returns tuple: (str or None, str or None).

        First is the activate script full path (or None if it's wasn't found) - conda's or venv's.

        Second is the env argument of the activate script (or None if not needed).
        """
        if p.isdir(p.join(self.local_root, 'conda-meta')):
            # check if we are installing to conda root:
            activate = self._check_if_conda_root(self.local_root)
            if activate:
                return activate, None

            # check if we are installing to default conda env location:
            #   `<conda_root>/envs/<local_root_basename>`
            ddot = p.dirname(self.local_root)
            activate = self._check_if_conda_root(p.dirname(ddot))
            if (p.basename(ddot) == 'envs') and activate: 
                return activate, p.basename(self.local_root)

            # check if we are running pip via `conda env create -f env.yaml`
            #   or user specified `CONDA_ROOT` env var himself:
            conda_root = os.environ.get('CONDA_ROOT')
            activate = self._check_if_conda_root(conda_root)
            if activate:
                if p.isabs(conda_root):
                    return activate, self.local_root

            # check if there is conda in the PATH:
            conda = self.find_target('conda')
            if conda is not None:
                conda_root = p.dirname(p.dirname(conda))
                activate = self._check_if_conda_root(conda_root)
                if activate:
                    return activate, self.local_root

            return None, self.local_root
        else:
            # check if we are installing to venv:
            activate = p.join(self.bin_folder_pyexe, self.ba('activate'))
            if p.isfile(activate):
                return activate, None

        return None, None
 
    def _check_if_conda_root(self, path):
        """
        Checks if provided path is conda root. It should have
        conda-meta folder, conda executable, activate shell script.

        Returns path to conda activate script or None.
        """
        if path is not None:
            if p.isdir(p.join(path, 'conda-meta')):
                conda = p.join(path,
                               p.basename(self.bin_folder_pyexe),
                               self.exe('conda'))
                # check if the file executable:
                if p.isfile(conda) and os.access(conda, os.X_OK):
                    activate = p.join(p.dirname(conda), self.ba('activate'))
                    if p.isfile(activate):
                        return p.abspath(activate)
        return None

    def create_desktop_shortcut(self, target, shortcut_name=None):
        """
        Creates a desktop shortcut to a target.

        Parameters
        ----------
        target : str
            The target to create a shortcut for, it can be a fully qualified
            file path ``/path/to/my_program`` or a simple application name 
            ``my_program``.
        shortcut_name : str=None
            Name of the shortcut without extension (``.lnk`` would be appended if needed).
            If None uses the target filename.

        Returns
        -------
        tuple (str, str, str or None)
            (shortcut_name, target_path, shortcut_file_path)
        """
        if not p.isdir(self.desktop_folder):
            msg = "Desktop folder '{}' not found.".format(self.desktop_folder)
            if self.raise_errors:
                raise ShortcutNoDesktopError(msg)
            elif self.error_log is not None:
                self.error_log.write(msg + '\n')
        else:
            return self.create_shortcut(target, self.desktop_folder, shortcut_name)

    def create_menu_shortcut(self, target, shortcut_name=None):
        """
        Creates a menu shortcut to a target.

        Parameters
        ----------
        target : str
            The target to create a shortcut for, it can be a fully qualified
            file path ``/path/to/my_program`` or a simple application name 
            ``my_program``.
        shortcut_name : str=None
            Name of the shortcut without extension (``.lnk`` would be appended if needed).
            If None uses the target filename.

        Returns
        -------
        tuple (str, str, str or None)
            (shortcut_name, target_path, shortcut_file_path)
        """
        if not p.isdir(self.menu_folder):
            msg = "Menu folder '{}' not found.".format(self.menu_folder)
            if self.raise_errors:
                raise ShortcutNoMenuError(msg)
            elif self.error_log is not None:
                self.error_log.write(msg + '\n')
        else:
            return self.create_shortcut(target, self.menu_folder, shortcut_name)

    @classmethod
    def _path_to_name(cls, path):
        """
        Takes three last items from the absolutized path and converts to
        name by replacing everything except `A-Za-z0-9` to `_`
        """
        dirs = '_'.join(p.abspath(path).split(os.sep)[-3:-1])
        return cls._ascii_name('{}__at__{}'.format(p.basename(path), dirs))

    @staticmethod
    def _ascii_name(name):
        w_unicode_name = re.sub(r'\W', '_', name)
        w_ascii_name = str(w_unicode_name.encode('utf-8'))[1:].strip('"').strip("'").replace('\\', '_')
        return w_ascii_name

    def _create_wrapped_shortcut(self, shortcut_name, target_path, shortcut_directory, activate_args=None):
        """
        Creates shell script wrapper for shortcut with activation.
        
        Providing activate_args optional argument switches to creation
        of shortcut to terminal with activated environment.
        
        Returns a tuple of (shortcut_name, target_path, shortcut_file_path)
        """
        if activate_args is None:
            activate, env = self.activate_args
            terminals = False
            name = self._path_to_name(target_path)
        else:
            activate, env = activate_args
            terminals = True
            target_path = None
            name = self._ascii_name(shortcut_name)

        wrapper_path = p.join(self.bin_folder_shcut, self.ba('shortcutter__' + name))
        if target_path or terminals:
            def r(path):
                if path is None:
                    return ''
                else:
                    return path.replace('"', r'\"').replace("'", r"\'")

            def call_batch(path):
                if path is None:
                    return ''
                elif os.name == 'nt' and (path.endswith('.bat') or path.endswith('.cmd')):
                    return 'call '
                else:
                    return ''

            script = (self._ACTIVATE_PROMPT if terminals else self._ACTIVATE).format(
                activate=r(activate) + ('" "' + r(env) if env else ''),
                executable=r(target_path),
                bin=r(p.dirname(activate)),
                call=call_batch(target_path)
            )
            with open(wrapper_path, 'w') as f:
                f.write(script)
                self._make_executable(wrapper_path)
        return self._create_shortcut_file(shortcut_name, wrapper_path, shortcut_directory)

    def create_shortcut(self, target, shortcut_directory, shortcut_name=None):
        """
        Creates a shortcut to a target.

        Parameters
        ----------
        target : str
            The target to create a shortcut for, it can be a fully qualified
            file path ``/path/to/my_program`` or a simple application name 
            ``my_program``.
        shortcut_directory : str
            The directory path where the shortcut should be created.
        shortcut_name : str=None
            Name of the shortcut without extension (``.lnk`` would be appended if needed).
            If None uses the target filename.

        Returns
        -------
        tuple (str, str, str or None)
            (shortcut_name, target_path, shortcut_file_path)
        """
        # Set the target path:
        target_path = self.find_target(target)

        # Check if target is dir or file:
        isdir = False
        if target_path:
            if p.isdir(target_path):
                isdir = True

        if not target_path and not self.exists:
            if target.endswith(os.sep):
                isdir = True
            target_path = p.abspath(target)

        # Set shortcut name:
        if shortcut_name is None:
            if isdir:
                shortcut_name = p.basename(target)
            else:
                # getting the file name and removing the extension:
                shortcut_name = p.splitext(p.basename(target))[0]

        # Create shortcut function:
        def create():
            if not target_path:
                raise ShortcutError(
                    "Target '{}' wasn't found or invalid target/exists ('{}') options combination.".format(target,
                                                                                                           self.exists))

            if isdir:
                return self._create_shortcut_to_dir(shortcut_name, target_path, shortcut_directory)

            elif self.activate:
                activate, env = self.activate_args
                if activate:
                    return self._create_wrapped_shortcut(shortcut_name, target_path, shortcut_directory)

                elif (not activate) and env:
                    raise ShortcutError('Shortcutter failed to find conda root (or activate script there). ' +
                                        'It searched in `../../` assuming default env location ' +
                                        '(`../` should have `envs` basename). ' +
                                        'Checked `CONDA_ROOT` environment variable. ' +
                                        'Searched `conda` executable in the PATH.')
            # Use simple shortcuts if self.activate=False or we are installing to common python installation:
            return self._create_shortcut_file(shortcut_name, target_path, shortcut_directory)

        ret = self._safe_create(create)
        return ret if (ret != 'error') else (shortcut_name, target_path, None)

    def _safe_create(self, create):
        """
        Switches shortcuts creation function error modes.

        :param create:
            function to call (without arguments)
        """
        if self.raise_errors:
            ret = create()
        else:
            try:
                ret = create()
            except Exception:
                ret = 'error'
                if self.error_log is not None:
                    self.error_log.write(''.join(traceback.format_exc()))
        return ret

    def create_shortcut_to_env_terminal(self, shortcut_name=None, shortcut_directory=None, desktop=True, menu=True):
        """
        Creates shortcuts for console (terminal) that
        has already activated the environment we are installing to
        (plus shortcut to root environment in case of conda).

        Parameters
        ----------
        shortcut_name : str=None
            Name of the shortcut without extension (``.lnk`` would be appended if needed).
            If None uses the target filename.
        shortcut_directory : str=None
            The directory path where the shortcuts should be created.
        desktop : bool=True
            Whether to create shortcuts on the desktop.
        menu : bool=True
            Whether to create shortcuts in the menu.

        Returns
        -------
        bool
            True if all operations were successful, False otherwise.
        """
        activate, env = self.activate_args
        if not activate:
            return
        if not shortcut_name:
            shortcut_name = 'Terminal at '

        ret = []
        for check, path, pref in [(desktop, self.desktop_folder, 'Desktop folder'),
                                  (menu, self.menu_folder, 'Menu folder'),
                                  (shortcut_directory is not None, shortcut_directory, 'Directory')]:
            if check:
                if not p.isdir(path):
                    msg = "{} '{}' not found.".format(pref, path)
                    if self.raise_errors:
                        raise ShortcutNoDesktopError(msg)
                    elif self.error_log is not None:
                        self.error_log.write(msg + '\n')
                        ret.append('error')
                else:
                    name = shortcut_name + p.basename(p.dirname(p.dirname(activate)))
                    ret.append(self._safe_create(
                        lambda: self._create_wrapped_shortcut(name, None, path, (activate, None))
                    ))
                    if env:
                        name = shortcut_name + p.basename(env)
                        ret.append(self._safe_create(
                            lambda: self._create_wrapped_shortcut(name, None, path, (activate, env))
                        ))
        return False if ('error' in ret) else True

    # should be overridden
    def _create_shortcut_to_dir(self, shortcut_name, target_path, shortcut_directory):
        raise ShortcutError("_create_shortcut_to_dir needs overriding")

    # should be overridden
    def _create_shortcut_file(self, shortcut_name, target_path, shortcut_directory):
        raise ShortcutError("_create_shortcut_file needs overriding")

    def makedirs(self, *args):
        """
        Recursively creates dirs if they don't exist.
        Utilizes ``self.raise_errors`` and ``self.error_log``.
        
        Parameters
        ----------
        \*args : str
            Multiple paths (str) for folders to create.

        Returns
        -------
        bool
            True on success False of failure.
        """
        ret = []
        for path in args:
            if not p.isdir(path):
                ret.append(self._safe_create(lambda: os.makedirs(path)))

        return False if ('error' in ret) else True

    def find_target(self, target):
        """
        Finds a file path for a target application.
        Single-worded targets like ``'app'`` are always searched in the PATH.
        You should prepend ``./app`` to tell that the file is in the CWD.

        Parameters
        ----------
        target : str
            The target to find, it can be a fully qualified
            file path ``/path/to/my_program`` or a simple application name 
            ``my_program``.

        Returns
        -------
        str or None
            Returns a single target file path or ``None`` if a path can't be found.
        """
        if target:
            if p.basename(target) == target:
                targets = self.search_for_target(target)
                if len(targets) > 0:
                    return p.abspath(targets[0])
                else:
                    return None
            elif p.isfile(target) or p.isdir(target):
                return p.abspath(target)
        return None

    def search_for_target(self, target):
        """
        Searches for a target application.

        Parameters
        ----------
        target : str
            The target to find.

        Returns
        -------
        list(str)
            Returns a list of potential target file paths, it no paths are found an empty list is returned.
        """
        # potential list of app paths
        target_paths = []

        # create list of potential directories
        paths = self._get_paths()

        # loop through each folder
        for path in paths:
            if p.exists(path):
                if p.isdir(path):
                    # get files in directory
                    for file_name in os.listdir(path):
                        file_path = p.join(path, file_name)
                        if p.isfile(file_path):
                            if self._is_file_the_target(target, file_name, file_path):
                                target_paths.append(file_path)
                else:
                    # its not a directory, is it the app we are looking for?
                    pass

        return target_paths

    # needs overriding
    def _is_file_the_target(self, target, file_name, file_path):
        raise ShortcutError("_is_file_the_target needs overriding")

    # needs overriding
    @staticmethod
    def _get_paths():
        raise ShortcutError("_get_paths needs overriding")



from os import path as p
from .exception import ShortcutError
from .linux import ShortCutterLinux
from tempfile import NamedTemporaryFile
import subprocess


def create_shortcut(shortcut_name, target_path, shortcut_directory, script):
    """
    Creates a MacOS app which opens an target_path (executable or folder) using AppleScript script

    Returns tuple (shortcut_name, target_path, shortcut_file_path)
    """
    shortcut_file_path = p.join(shortcut_directory, shortcut_name + ".app")

    # create the AppleScript script
    sf = NamedTemporaryFile(mode="w")
    sf.write(script)
    sf.flush()

    # compile the script into an application
    result = subprocess.run(["osacompile", "-o", shortcut_file_path, sf.name], stdout=subprocess.PIPE,
                            stderr=subprocess.PIPE)
    if len(result.stderr):
        raise ShortcutError("Error occured creating app - {}".format(str(result.stderr)))

    sf.close()

    return shortcut_name, target_path, shortcut_file_path


class ShortCutterMacOS(ShortCutterLinux):
    @staticmethod
    def _get_desktop_folder():
        return p.join(p.expanduser('~'), 'Desktop')

    @staticmethod
    def _get_menu_folder():
        return p.join('/', 'Applications') 

    def _create_shortcut_to_dir(self, shortcut_name, target_path, shortcut_directory):
        """
        Creates a MacOS app which opens a folder via finder
        """
        return create_shortcut(shortcut_name, target_path, shortcut_directory,
                               'tell application "Finder"\n' +
                               'open POSIX file "{}"\n'.format(target_path) +
                               'end tell\n')

    def _create_shortcut_file(self, shortcut_name, target_path, shortcut_directory):
        """
        Creates a MacOS app which opens an executable via the terminal
        """
        return create_shortcut(shortcut_name, target_path, shortcut_directory,
                               'tell application "Terminal"\n' +
                               'activate\n' +
                               'do script "{}"\n'.format(target_path) +
                               'end tell\n')



import sys
import os
from os import path as p
import stat
from .base import ShortCutter

ACTIVATE = """#!/bin/bash
export PYTHONNOUSERSITE=1
source "{activate}"
"{executable}" "$@"
source "{bin}/deactivate"
"""

ACTIVATE_PROMPT = """#!/bin/bash
bash --rcfile <(echo 'export PATH="{bin}:$PATH"; export PYTHONNOUSERSITE=1; source "{activate}"; cd $HOME')
"""


class ShortCutterLinux(ShortCutter):
    @staticmethod
    def _get_desktop_folder():
        import subprocess
        try:
            return subprocess.check_output(['xdg-user-dir',
                                            'DESKTOP']).decode('utf-8').strip()
        except subprocess.CalledProcessError:
            return p.join(p.expanduser('~'), 'Desktop')

    @staticmethod
    def _get_menu_folder():
        return p.join(p.expanduser('~'), '.local', 'share', 'applications')

    @staticmethod
    def _get_bin_folder_pyexe():
        return p.dirname(sys.executable)

    @staticmethod
    def _get_local_root():
        return p.dirname(p.dirname(sys.executable))

    @staticmethod
    def _get_activate_wrapper_templates():
        return ACTIVATE, ACTIVATE_PROMPT

    @staticmethod
    def _make_executable(file_path):
        st = os.stat(file_path)
        os.chmod(file_path, st.st_mode | stat.S_IEXEC)

    def _create_shortcut_to_dir(self, shortcut_name, target_path, shortcut_directory):
        """
        Creates a Linux shortcut file to executable.
        """
        return self._create_shortcut_linux(shortcut_name, target_path, shortcut_directory,
                                           '[Desktop Entry]\n' +
                                           'Name={}\n'.format(shortcut_name) +
                                           'Type=Application\n' +
                                           'Path={}\n'.format(target_path) +
                                           'Exec=xdg-open "{}"\n'.format(target_path) +
                                           'Icon=system-file-manager.png\n')

    def _create_shortcut_file(self, shortcut_name, target_path, shortcut_directory):
        """
        Creates a Linux shortcut file to folder.
        """
        return self._create_shortcut_linux(shortcut_name, target_path, shortcut_directory,
                                           '[Desktop Entry]\n' +
                                           'Name={}\n'.format(shortcut_name) +
                                           'Type=Application\n' +
                                           'Exec="{}" %F\n'.format(target_path) +
                                           'Terminal=true\n')

    def _create_shortcut_linux(self, shortcut_name, target_path, shortcut_directory, script):
        """
        Creates a Linux shortcut file using .desktop file script

        Returns tuple (shortcut_name, target_path, shortcut_file_path)
        """
        shortcut_file_path = p.join(shortcut_directory, shortcut_name + '.desktop')

        with open(shortcut_file_path, "w") as shortcut:
            shortcut.write(script)
            self._make_executable(shortcut_file_path)

        return shortcut_name, target_path, shortcut_file_path

    def _is_file_the_target(self, target, file_name, file_path):
        match = False
        if file_name == target:
            # is the file executable
            if os.access(file_path, os.X_OK):
                match = True
            else:
                match = False
        return match

    @staticmethod
    def _get_paths():
        """
        Gets paths from the PATH environment variable and
        prepends the `<Python>/bin` directory.

        Returns a list of paths.
        """
        return [p.dirname(sys.executable)] + os.environ['PATH'].split(os.pathsep) + [p.join(p.expanduser('~'), '.local', 'bin')]



import sys
import site
import os
from os import path as p

# import win32com
# this often fails due to unable to find DLLs
# so dynamically change the path if required
try:
    import win32com
except ImportError as e:
    if "DLL load failed:" in str(e):
        path = p.join(p.split(sys.executable)[0], "Lib", "site-packages", "pywin32_system32")
        os.environ["PATH"] = os.environ["PATH"] + ";" + path
        try:
            import win32com
        except ImportError as ee:
            dll = os.listdir(path)
            dll = [p.join(path, _) for _ in dll if "dll" in _]
            raise ImportError("{}\n Failed to import win32com, due to missing DLL:\n".format(ee) + "\n".join(dll))
    else:
        raise e

from win32com.client import Dispatch
from .base import ShortCutter

shell = Dispatch('WScript.Shell')

ACTIVATE = r"""@echo off
set PYTHONNOUSERSITE=1
call "{activate}"
{call}"{executable}" %*
call "{bin}\deactivate.bat"
"""

ACTIVATE_PROMPT = """@echo off
set "PATH={bin};%PATH%"
set PYTHONNOUSERSITE=1
call "{activate}"
set "u=chcp 65001 && set PYTHONIOENCODING=utf-8"
set "a=chcp 1252 && set PYTHONIOENCODING="
set "b=set LANG=C.UTF-8 && set PYTHONIOENCODING=utf-8 && "%PROGRAMFILES%\Git\git-bash.exe""
cd /d %USERPROFILE%
cmd /k
"""

FOLDER_SHORTCUT = r"""@echo off
if exist "{path}\" (
    cd /d "{path}"
    start .
) else (
    echo Folder doesn't exist: "{path}"
    pause
)
"""


class ShortCutterWindows(ShortCutter):
    def _set_win_vars(self):
        self._executable_file_extensions = [ext.upper() for ext in os.environ['PATHEXT'].split(os.pathsep)]

    @staticmethod
    def _get_desktop_folder():
        return shell.SpecialFolders("Desktop")

    @staticmethod
    def _get_menu_folder():
        return shell.SpecialFolders("Programs")

    @staticmethod
    def _get_bin_folder_pyexe():
        return p.join(p.dirname(sys.executable), "Scripts")

    @staticmethod
    def _get_local_root():
        return p.dirname(sys.executable)

    @staticmethod
    def _get_activate_wrapper_templates():
        return ACTIVATE, ACTIVATE_PROMPT

    @staticmethod
    def _make_executable(file_path):
        pass

    def _create_shortcut_to_dir(self, shortcut_name, target_path, shortcut_directory):
        return self._create_shortcut_win(shortcut_name, target_path, shortcut_directory, True)

    def _create_shortcut_file(self, shortcut_name, target_path, shortcut_directory):
        return self._create_shortcut_win(shortcut_name, target_path, shortcut_directory)

    def _create_shortcut_win(self, shortcut_name, target_path, shortcut_directory, folder=False):
        """
        Creates a Windows shortcut file.

        Returns tuple (shortcut_name, target_path, shortcut_file_path)
        """
        icon = None
        if not folder:
            shortcut_target_path = target_path
            working_directory = p.dirname(target_path)
            ext = p.splitext(target_path)[1].upper()
            if ext in self._executable_file_extensions:
                icon = r'%SystemRoot%\System32\imageres.dll,11'

        elif not p.isdir(target_path):
            # create a bat script that opens the folder:
            
            wrapper_path = p.join(self.bin_folder_shcut, self.ba('shortcutter__dir__' + self._path_to_name(target_path)))
            with open(wrapper_path, 'w') as f:
                f.write(FOLDER_SHORTCUT.format(path=target_path))
            shortcut_target_path = wrapper_path
            working_directory = self.bin_folder_shcut
            icon = r'%SystemRoot%\explorer.exe,0'

        else:
            shortcut_target_path = target_path
            working_directory = target_path

        shortcut_file_path = p.join(shortcut_directory, shortcut_name + ".lnk")
        shortcut = shell.CreateShortCut(shortcut_file_path)
        shortcut.Targetpath = shortcut_target_path
        shortcut.WorkingDirectory = working_directory
        shortcut.Description = "Shortcut to" + p.basename(target_path)
        if icon:
            shortcut.IconLocation = icon
        shortcut.save()

        return shortcut_name, target_path, shortcut_file_path

    def _is_file_the_target(self, target, file_name, file_path):
        match = False
        # does the target have an extension?
        target_ext = p.splitext(target)[1]
        # if so, do a direct match
        if target_ext:
            if file_name.lower() == target.lower():
                match = True
        # no extension, compare the target to the file_name for each executable file extension
        else:
            for extension in self._executable_file_extensions:
                if file_name.lower() == (target + extension).lower():
                    match = True
        return match

    @staticmethod
    def _get_paths():
        """
        Gets paths from the PATH environment variable and 
        prepends `<Python>`, `<Python>\Scripts`, `<Python>\Library\bin` directories.

        Returns a list of paths.
        """
        root = p.dirname(sys.executable)
        user_root = p.dirname(site.USER_SITE)
        return [root,
                p.join(root, 'Scripts'),
                p.join(root, 'Library', 'bin')
               ] + os.environ['PATH'].split(os.pathsep) + [p.join(user_root, 'Scripts')]



import sys
from setuptools import setup, find_packages
from os import path as p
import versioneer
import io

if sys.version_info[0] == 2:
    if not sys.version_info >= (2, 7):
        raise ValueError('This package requires Python 2.7 or newer')
elif sys.version_info[0] == 3:
    if not sys.version_info >= (3, 3):
        raise ValueError('This package requires Python 3.3 or newer')
else:
    raise ValueError('Unrecognized major version of Python')

here = p.abspath(p.dirname(__file__))
with io.open(p.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()


setup(
    name='shortcutter',
    version = versioneer.get_version(),
    cmdclass=versioneer.get_cmdclass(),
    description = 'Easy cross-platform creation of shortcuts supporting virtual and Anaconda environments (fork of Shortcut)',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url = 'https://github.com/kiwi0fruit/shortcutter',
    author = "Martin O'Hanlon, Peter Zagubisalo",
    author_email = 'peter.zagubisalo@gmail.com',
    license= 'MIT',
    classifiers=[
        "Development Status :: 3 - Alpha",
        # "Development Status :: 4 - Beta",
        # "Development Status :: 5 - Production/Stable",
        "Intended Audience :: End Users/Desktop",
        "Topic :: Desktop Environment",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Environment :: MacOS X",
        "Environment :: Win32 (MS Windows)",
    ],
    packages = find_packages(exclude=['docs', 'tests']),
    # install_requires = ['pywin32;platform_system=="Windows"'],
    extras_require = {
        ':sys_platform == "win32"': ['pywin32',],
    },
    entry_points={
        'console_scripts': [
            'shortcutter = shortcutter:main'
        ]},
    zip_safe=False
)
